@@ -34,10 +34,6 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * These functions can be shared between SAM, BAM and CRAM file
  * formats as all three internally use the same string encoding for
  * header fields.
- *
- * Consider using the scram() generic API and calling
- * scram_get_header() to obtain the format-specific pointer to the
- * SAM_hdr struct.
  */ 
 
 /*
@@ -228,11 +224,7 @@ SAM_hdr *sam_hdr_new(void);
  * Returns a SAM_hdr struct on success (free with sam_hdr_free());
  *         NULL on failure
  */
-#ifdef SAMTOOLS
 SAM_hdr *sam_hdr_parse_(const char *hdr, int len);
-#else
-SAM_hdr *sam_hdr_parse(const char *hdr, int len);
-#endif
 
 
 /*! Produces a duplicate copy of hdr and returns it.
