@@ -327,11 +327,9 @@ int kftp_connect_file(knetFile *fp)
 	kftp_pasv_prep(fp);
     kftp_send_cmd(fp, fp->size_cmd, 1);
 #ifndef _WIN32
-    if ( sscanf(fp->response,"%*d %lld", &file_size) != 1 )
-    {
-        fprintf(stderr,"[kftp_connect_file] %s\n", fp->response);
-        return -1;
-    }
+    // If the file does not exist, the response will be "550 Could not get file
+    // size". Be silent on failure, hts_idx_load can be trying the existence of .csi or .tbi.
+    if ( sscanf(fp->response,"%*d %lld", &file_size) != 1 ) return -1;
 #else
 	const char *p = fp->response;
 	while (*p != ' ') ++p;
@@ -413,7 +411,7 @@ int khttp_connect_file(knetFile *fp)
 	l += sprintf(buf + l, "GET %s HTTP/1.0\r\nHost: %s\r\n", fp->path, fp->http_host);
     l += sprintf(buf + l, "Range: bytes=%lld-\r\n", (long long)fp->offset);
 	l += sprintf(buf + l, "\r\n");
-	if ( netwrite(fp->fd, buf, l) != l ) return -1;
+	if ( netwrite(fp->fd, buf, l) != l ) { free(buf); return -1; }
 	l = 0;
 	while (netread(fp->fd, buf + l, 1)) { // read HTTP header; FIXME: bad efficiency
 		if (buf[l] == '\n' && l >= 3)
@@ -422,6 +420,7 @@ int khttp_connect_file(knetFile *fp)
 	}
 	buf[l] = 0;
 	if (l < 14) { // prematured header
+		free(buf);
 		netclose(fp->fd);
 		fp->fd = -1;
 		return -1;
