@@ -32,6 +32,82 @@ DEALINGS IN THE SOFTWARE.  */
 
 #include "htslib/sam.h"
 
+typedef struct hts_opt {
+    enum cram_option opt;
+    union {
+        int i;
+        char *s;
+    } val;
+    struct hts_opt *next;
+} hts_opt;
+
+/*
+ * Parses arg and appends it to the option list.
+ * Returns 0 on success;
+ *        -1 on failure.
+ */
+int add_option(hts_opt **opts, char *arg) {
+    hts_opt *o, *t;
+    char *cp;
+
+    if (!(cp = strchr(arg, '=')))
+        cp = "1"; // assume boolean
+    else
+        *cp++ = 0;
+
+    if (!(o =  malloc(sizeof(*o))))
+        return -1;
+
+    if (strcmp(arg, "DECODE_MD") == 0)
+        o->opt = CRAM_OPT_DECODE_MD, o->val.i = atoi(cp);
+    else if (strcmp(arg, "VERBOSITY") == 0)
+        o->opt = CRAM_OPT_VERBOSITY, o->val.i = atoi(cp);
+    else if (strcmp(arg, "SEQS_PER_SLICE") == 0)
+        o->opt = CRAM_OPT_SEQS_PER_SLICE, o->val.i = atoi(cp);
+    else if (strcmp(arg, "SLICES_PER_CONTAINER") == 0)
+        o->opt = CRAM_OPT_SLICES_PER_CONTAINER, o->val.i = atoi(cp);
+    else if (strcmp(arg, "EMBED_REF") == 0)
+        o->opt = CRAM_OPT_EMBED_REF, o->val.i = atoi(cp);
+    else if (strcmp(arg, "NO_REF") == 0)
+        o->opt = CRAM_OPT_NO_REF, o->val.i = atoi(cp);
+    else if (strcmp(arg, "IGNORE_MD5") == 0)
+        o->opt = CRAM_OPT_IGNORE_MD5, o->val.i = atoi(cp);
+    else if (strcmp(arg, "USE_BZIP2") == 0)
+        o->opt = CRAM_OPT_USE_BZIP2, o->val.i = atoi(cp);
+    else if (strcmp(arg, "USE_RANS") == 0)
+        o->opt = CRAM_OPT_USE_RANS, o->val.i = atoi(cp);
+    else if (strcmp(arg, "USE_LZMA") == 0)
+        o->opt = CRAM_OPT_USE_LZMA, o->val.i = atoi(cp);
+    else if (strcmp(arg, "REFERENCE") == 0)
+        o->opt = CRAM_OPT_REFERENCE, o->val.s = cp;
+    else if (strcmp(arg, "VERSION") == 0)
+        o->opt = CRAM_OPT_VERSION, o->val.s =cp;
+    else if (strcmp(arg, "MULTI_SEQ_PER_SLICE") == 0)
+        o->opt = CRAM_OPT_MULTI_SEQ_PER_SLICE, o->val.i = atoi(cp);
+    else if (strcmp(arg, "NTHREADS") == 0)
+        o->opt = CRAM_OPT_NTHREADS, o->val.i = atoi(cp);
+    else if (strcmp(arg, "REQUIRED_FIELDS") == 0)
+        o->opt = CRAM_OPT_REQUIRED_FIELDS, o->val.i = strtol(cp, NULL, 0);
+    else {
+        fprintf(stderr, "Unknown option '%s'\n", arg);
+        free(o);
+        return -1;
+    }
+
+    o->next = NULL;
+
+    if (*opts) {
+        t = *opts;
+        while (t->next)
+            t = t->next;
+        t->next = o;
+    } else {
+        *opts = o;
+    }
+
+    return 0;
+}
+
 int main(int argc, char *argv[])
 {
     samFile *in;
@@ -43,8 +119,9 @@ int main(int argc, char *argv[])
     htsFile *out;
     char modew[8];
     int r = 0, exit_code = 0;
+    hts_opt *in_opts = NULL, *out_opts = NULL, *last = NULL;
 
-    while ((c = getopt(argc, argv, "IbDCSl:t:")) >= 0) {
+    while ((c = getopt(argc, argv, "IbDCSl:t:i:o:")) >= 0) {
         switch (c) {
         case 'S': flag |= 1; break;
         case 'b': flag |= 2; break;
@@ -53,10 +130,12 @@ int main(int argc, char *argv[])
         case 'l': clevel = atoi(optarg); flag |= 2; break;
         case 't': fn_ref = optarg; break;
         case 'I': ignore_sam_err = 1; break;
+        case 'i': if (add_option(&in_opts,  optarg)) return 1; break;
+        case 'o': if (add_option(&out_opts, optarg)) return 1; break;
         }
     }
     if (argc == optind) {
-        fprintf(stderr, "Usage: samview [-bSCSI] [-l level] <in.bam>|<in.sam>|<in.cram> [region]\n");
+        fprintf(stderr, "Usage: samview [-bSCSI] [-l level] [-o option=value] <in.bam>|<in.sam>|<in.cram> [region]\n");
         return 1;
     }
     strcpy(moder, "r");
@@ -95,6 +174,15 @@ int main(int argc, char *argv[])
             cram_set_option(out->fp.cram, CRAM_OPT_REFERENCE, NULL);
     }
 
+    // Process any options; currently cram only.
+    for (; in_opts;  in_opts = (last=in_opts)->next, free(last)) {
+        hts_set_opt(in,  in_opts->opt,  in_opts->val);
+        if (in_opts->opt == CRAM_OPT_REFERENCE)
+            hts_set_opt(out,  in_opts->opt,  in_opts->val);
+    }
+    for (; out_opts;  out_opts = (last=out_opts)->next, free(last))
+        hts_set_opt(out, out_opts->opt,  out_opts->val);
+
     sam_hdr_write(out, h);
     if (optind + 1 < argc && !(flag&1)) { // BAM input and has a region
         int i;
